import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.utils.*;
import com.haxepunk.graphics.*;


enum CreatureState {
	STANDING;
	FALLING;
	CRAWLING;
	CONVEX_CORNER;		// Animation states
	CONCAVE_CORNER;
	TOCRAWL;
	TOSTAND;
	BITE;				// Into feeding
	FEEDING;			// feed idle
	FED;				// exit feeding
	DEAD;
}


class Creature extends Entity
{

	public var health: Float = 100;			// Current health
	public var maxHealth: Float = 100;		// Current max health
	public var state: CreatureState;		// Control state
	public var walls: Int;					// Which walls we are touching (bitmask top,btm,lt,rt);
	public var wall: Int;					// Wall we are currently attached to
	public var vy: Float= 0;				// Falling velocity
	public var vx: Float = 0;				// x velocity for falling
	public var speed: Float = 10;			// Movement speed

	public var lunch: Human;

	public var sprite: Spritemap;

	public function new(x:Float, y:Float) {
		super(x, y);

		Input.define("up", [Key.UP]);
		Input.define("dn", [Key.DOWN]);
		Input.define("lt", [Key.LEFT]);
		Input.define("rt", [Key.RIGHT]);
		Input.define("action",  [Key.SPACE, Key.X]);

		sprite = new Spritemap("graphics/creature.png", 155, 145, endAnimation);
		sprite.add("idle", [23,24,25,26,27,28], 10, true);
		sprite.add("run", [29,30,31,32,33,34,35,36,37,38], 15, true);
		sprite.add("crouch", [17,18,19,20,21,22], 30, false);
		sprite.add("getup",  [22,21,20,19,18,17], 30, false);
		sprite.add("crouched", [22], 10, false);
		sprite.add("crawl", [5,6,7,8,9,10,11,12,13,14,15,16], 20, true);
		sprite.add("bite", [0,1,2,3,4], 20, false);
		sprite.add("endbite", [4,3,2,1,0], 20, false);
		sprite.centerOrigin();
		graphic = sprite;

		sprite.play("idle");
		setHitbox(50, 100, 25, 40);

		state = STANDING;
	}

	public override function moveCollideY(e:Entity): Bool {
		var r:Bool = super.moveCollideY(e);
		if(r) {
			if(vy>0 && state==FALLING) state = STANDING;
			if(vy<0) walls |= 0x1;	// Ceiling
			if(vy>0) walls |= 0x2;	// Floor
			vy = 0;
		}
		return r;
	}
	public override function moveCollideX(e:Entity): Bool {
		var r:Bool = super.moveCollideX(e);
		if(r)  {
			if(vx < 0) walls |= 0x4;	// Left
			if(vx > 0) walls |= 0x8;	// Right
			vx = 0;
		}
		return r;
	}

	public function jump() {
		vx *= 3.0;
		vy = Math.abs(vx)*0.5 - 40;
	}

	public function bite(target:Human) {
		lunch = target;
		lunch.changeState(LUNCH);
		if(state == CRAWLING) sprite.play("getup");
		else sprite.play("bite");
		state = BITE;
	}

	public function endAnimation() {
		if(state == TOCRAWL) state = CRAWLING;
		if(state == TOSTAND) state = STANDING;
		if(state == FED) state = STANDING;
		if(state == BITE) {
			if(sprite.currentAnim == "getup") sprite.play("bite");
			else state = FEEDING;
		}
	}

	public function setWall(newWall:Int) {
		sprite.centerOrigin();
		switch(newWall) {
		case 0:
			setHitbox(50, 100, 25, 40);
			sprite.angle = 0;
		case 1:
			setHitbox(100, 50, 50, 40);
			sprite.originY += 20;
			sprite.angle = 180;
		case 2:
			setHitbox(100, 50, 50, -10);
			sprite.angle = 0;
		case 3: 
			setHitbox(50, 100, 50, 40);
			if(wall==0) x += 25;
			sprite.originY += 10;
			sprite.angle = -90;
		case 4:
			setHitbox(50, 100,  0, 40);
			if(wall==0) x -= 25;
			sprite.originY += 10;
			sprite.angle = 90;
		}
		wall = newWall;
	}

	public override function update() {
		var mx:Float=0, my:Float=0;
		if(Input.check("lt")) mx = -speed;
		if(Input.check("rt")) mx = speed;
		if(Input.check("up")) my = -speed;
		if(Input.check("dn")) my = speed;

		// Drain health
		health = Math.max(0, health - 0.08);


		var solid: SolidType = collideTypes(["duct"],x,y)==null? ["solid","duct"]: ["solid"];

		switch(state) {
			case STANDING:
				if(mx==0 || HXP.sign(mx)!=HXP.sign(vx)) vx *= 0.8;
				vx = HXP.clamp(vx + mx * 0.06, -speed, speed);
				if(Input.pressed("up")) jump();
				else vy += 3;
				moveBy(vx, vy, solid);
				if(vy!=0) state = FALLING;

				if(Math.abs(vx) > 1) sprite.play("run");
				else sprite.play("idle");
				if(vx>1) sprite.flipped = false;
				else if(vx<-1) sprite.flipped = true;

			case FALLING:
				vy += 3;
				vx = HXP.clamp(vx + mx * 0.01, -speed*3, speed*3);
				moveBy(vx, vy, solid);
				sprite.play("idle"); // dont have fall animations yet
				sprite.flipped = vx < 0;
				setWall(0);

			case CRAWLING:
				if(wall==1 || wall==2) {	// Ceiling | Floor
					if(mx==0 || HXP.sign(mx)!=HXP.sign(vx)) vx *= 0.8;
					vx = HXP.clamp(vx + mx * 0.06, -speed, speed);
					vy = 0;
					moveBy(vx, 0, ["solid"]);
				}

				else if(wall==3 || wall==4) {
					if(my==0 || HXP.sign(my)!=HXP.sign(vy)) vy *= 0.8;
					vy = HXP.clamp(vy + my * 0.06, -speed, speed);
					vx = 0;
					moveBy(0, vy, ["solid"]);
				}
				
				if(Math.abs(vx) > 1 || Math.abs(vy)> 1) {
					sprite.flipped = (vx<0 || vy<0) != (wall==1 || wall==4);
					sprite.play("crawl");
				} else sprite.play("crouched");

				// Jump
				switch(wall) {
				case 1: if(Input.pressed("dn")) state = FALLING;
				case 2: if(Input.pressed("up")) state = TOSTAND;
				case 3: if(Input.pressed("rt")) { vx = 10; state = FALLING; }
				case 4: if(Input.pressed("lt")) { vx = -10; state = FALLING; }
				}

				// Fall
				if((walls & (1<<wall-1)) == 0) state = FALLING;

				// Check there is space to fall off ceiling
				if(wall==1 && state == FALLING && scene.collideRect("solid",x-25,y-40,50,100)!=null) {
					state = CRAWLING;
					setWall(2);
					y -= 50;
					moveBy(0, 50, ["solid"]);
					walls |= 0x2;
				}

				if(state == FALLING && wall>2) {
					if(wall == 3) { x -= 25; sprite.flipped = false; }
					if(wall == 4) { x += 25; sprite.flipped = true; }
					setWall(0);
				}


				// ToDo: Climbing round convex points
				// when KeyPress into the surface
				// check point collision on head end
				// Change state: CORNER



				// Hack for entering floor grates
				if(wall==2 && Input.pressed("dn") && scene.collideRect("solid", x-25,y-30,50,100)==null) {
					setWall(0);
					state = FALLING;
					++y;
					walls = 0;
				}

			case TOCRAWL:
				if(mx==0 || HXP.sign(mx)!=HXP.sign(vx)) vx *= 0.8;
				vx = HXP.clamp(vx + mx * 0.06, -speed, speed);
				setWall(2);
				// Check there is room
				if(collideTypes("solid", x, y)!=null) {
					setWall(0);
					state = STANDING;
				} else {
					sprite.play("crouch");
				}

			case TOSTAND:
				if(mx==0 || HXP.sign(mx)!=HXP.sign(vx)) vx *= 0.8;
				vx = HXP.clamp(vx + mx * 0.06, -speed, speed);
				sprite.play("getup");
				setWall(0);
				
				// Check there is room
				if(collideTypes("solid", x, y)!=null) {
					trace("Cant stand");
					state = CRAWLING;
					setWall(1);
					walls |= 0x1;
					y += 50;
					moveBy(0, -50, ["solid"]);
					sprite.play("crouched");
				}
				


			case CONCAVE_CORNER:
				// Just run animation, then switch wall
				state = CRAWLING;

			case CONVEX_CORNER:
				// Automatically move forward/back to the right place for animation
				// Play animation
				// Move character to target wall
				// Update wall state
				// State back to climbing

			case BITE:
				vx = 0;
				return;
			case FEEDING:
				health = Math.min(100, health+1);
				lunch.chomp(2);
				if(!Input.check("action") || lunch.state == DEAD) {
					sprite.play("endbite");
					state = FED;
				}
				return;
			case FED:
				return;

			default:

		}

		// Bitey bite
		if(Input.pressed("action") && (state == STANDING || (state == CRAWLING && wall==2))) {
			var off: Float = sprite.flipped? -20: 20;
			var t:Entity = collideTypes(["character"], x+off, y);
			if(t!=null && (t.x>x != sprite.flipped)) bite(cast t);
		}

		// Attach to walls - use CONCAVE_CORNER animation state
		var attach: Int = 0;
		if((walls & 0x1)>0 && Input.check("up")) attach = 1;
		if((walls & 0x2)>0 && Input.check("dn")) attach = 2;
		if((walls & 0x4)>0 && Input.check("lt")) attach = 3;
		if((walls & 0x8)>0 && Input.check("rt")) attach = 4;
		if(attach!=wall && attach > 0) {
			if(attach == 2 && state == STANDING) state = TOCRAWL;
			else if(state==CRAWLING) state = CONCAVE_CORNER;
			else state = CRAWLING;
			// Check there is room
			var last: Int = wall;
			setWall(attach);
			if(last > 0 && collideTypes(["solid"], x, y)!=null) setWall(last);
		}


		// Detect away from walls
		if((walls & 0x1)>0 && collideTypes(solid, x, y-1)==null) walls &= ~0x1; 
		if((walls & 0x2)>0 && collideTypes(solid, x, y+1)==null) walls &= ~0x2; 
		if((walls & 0x4)>0 && collideTypes(solid, x-1, y)==null) walls &= ~0x4; 
		if((walls & 0x8)>0 && collideTypes(solid, x+1, y)==null) walls &= ~0x8; 

	}

}

