import com.haxepunk.HXP;
import com.haxepunk.Sfx;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;

class Door extends Entity
{

	public var locked: Bool;	// Does the door automatically open
	public var open: Bool;		// Is the door open?
	public var moving: Int;		// Opening / Closing
	public var amount: Float;	// Open amount

	public var speed: Int = 24;
	public var sensor: Int = 100;

	private var canvas: Canvas;
	private var upper: Stamp;
	private var lower: Stamp;

	private var sound: Sfx;

	public function new(x:Float, y:Float, locked:Bool=false) {
		super(x, y);

		upper = new Stamp("graphics/door_upper.png");
		lower = new Stamp("graphics/door_lower.png");
		//canvas = new Canvas(upper.width, upper.height);
		canvas = new Tilemap("graphics/tiles.png", upper.width, upper.height, 32,32);
		graphic = canvas;
		setHitboxTo(canvas);
		type = "solid";
		amount = 0;
		open = false;
		updateCanvas();

		sound = new Sfx("audio/door.mp3");
	}

	public function updateCanvas() {
		var m: Float = amount * canvas.height * 0.6;

		// Clear
		HXP.rect.setTo(0, 0, canvas.width, canvas.height);
		canvas.fill(HXP.rect, 0, 0);
		// Draw parts
		canvas.drawGraphic(0, Std.int(-m), upper);
		canvas.drawGraphic(0, Std.int(m), lower);
	}

	public override function update() {

		// Move door
		if(moving!=0) {
			amount = HXP.clamp(amount + speed / canvas.height * moving, 0, 1);
			if(amount == Math.max(0,moving)) moving = 0;	// Stop
			open = amount == 1;
			type = open? "open": "solid";
			updateCanvas();
		}

		// Sense stuff
		if(!locked) {
			var detect:Bool = scene.collideRect("character", x-sensor, bottom-50, width+2*sensor, 50)!=null;
			if(!detect && collideTypes(["character"], x, y)!=null) detect = true;
			if(detect && !open) moving = 1;
			if(!detect && amount>0) moving = -1;
			if((moving == -1 && amount == 1) || moving==1 && amount==0) sound.play(1,0,false);
		}

	}

}

