import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;
import com.haxepunk.utils.*;
import flash.geom.Point;


enum HumanState {
	IDLE;
	INVESTIGATE;
	PATROL;
	FLEE;
	COWER;

	LUNCH;
	DEAD;
}

class Human extends Entity
{

	public var health: Float = 100;	

	public var patrolDirection: Int = 1;
	public var direction: Int = 1;

	public var state: HumanState;
	public var stateTime: Int;

	public var target: Entity;
	public var lastSeen: Point = new Point(0,0);
	public var lastTime: Int = 0;
	public var seeTime: Int = 0;
	public var tagged: Array<Human> = [];
	
	public var vx: Float = 0;
	public var vy: Float = 0;


	public var text: String;
	public var textTime: Int = 0;

	public function new(x:Float, y:Float) {
		super(x, y);
		type = "character";
		vy = 10;
		state = IDLE;
	}

	public function say(s:String, over:Bool) {
		if(!over && text!=null) return;
		text = s;
		textTime = 100;
	}

	public function changeState(s:HumanState) {
		stateTime = 0;
		state = s;
	}

	public function canSee(e:Entity) : Bool {
		var d:Float = distanceFrom(e);
		if(d > 2000) return false;
		if(direction * (e.x-x) / d < 0.5) return false;
		return scene.collideLine("solid", Std.int(x), Std.int(y-40), Std.int(e.x), Std.int(e.y), 32) == null;
	}

	public override function update() {
		if(state == LUNCH || state == DEAD) return;

		if(vx!=0 || vy!=0) moveBy(vx, vy, ["solid", "duct"]);
		++stateTime;

		if(text!=null && --textTime==0) text = null;

		// Sense
		var dead: Array<Human> = [];
		var creature:Creature = cast(scene, Level).player;
		scene.getType("dead", dead);
		for(h in dead) {
			if(tagged.indexOf(h)<0 && canSee(h)) {
				target = h;
				changeState(INVESTIGATE);
			}
		}
		// See player
		if(canSee(creature)) {
			target = creature;
			lastSeen.setTo(creature.x, creature.y);
			lastTime = 0;
			++seeTime;
			if(seeTime > 16 && collideTypes(["solid"], x-direction*50,y)==null) changeState(FLEE);
			if(seeTime == 2) say("Huh?", false);
			if(state == INVESTIGATE) changeState(IDLE); // stop error
		} else {
			++lastTime;
			seeTime = 0;
		}


		// State effects
		switch(state) {
		case IDLE:
			if(stateTime > 400) {
				direction = -direction;
				stateTime = Math.floor(HXP.random * 300);
			}

		case PATROL:
			direction = patrolDirection;
			vx = direction * 1;
			if(collideTypes(["solid","duct","patrol"], x+direction*80, y)!=null) {
				patrolDirection = -patrolDirection;
			}
		case INVESTIGATE:
			if(stateTime==1) say("Whats that?", true);
			direction = HXP.sign(target.x-x);
			vx = direction * 6;
			if(distanceFrom(target) < 80) {
				say("He's dead!", true);
				vx = 0;
				tagged.push(cast target);
				target = null;
				changeState(COWER);
			}

		case FLEE:
			if(stateTime==1) say("aaaaargh", true);
			direction = HXP.sign(x-lastSeen.x);
			vx = direction * 6;
			if(collideTypes(["solid","duct","character"], x+direction*50, y)!=null) changeState(COWER);
			else if(lastTime > 100) changeState(COWER);

		case COWER:
			vx *= 0.8;
			if(stateTime == 1 || lastTime < 2) direction = HXP.sign(lastSeen.x-x);
			else if(stateTime&0x3f==0) direction = -direction;
			if(lastTime > 300 && stateTime > 200) changeState(PATROL);

		default:
		}
	}

	public function chomp(t:Float) {
		health -= t;
		if(health <= 0) die();
		vx = vy = 0;
	}

	public function die() {
		state = DEAD;
		type = "dead";
	}

	

	public override function render() {
		super.render();
		if(text != null) {
			Draw.text(text, x-40, y-80, {size:12});
		}
	}


}

