import com.haxepunk.HXP;
import com.haxepunk.Scene;
import com.haxepunk.Entity;
import com.haxepunk.Sfx;
import com.haxepunk.graphics.*;
import com.haxepunk.utils.*;


enum GameState {
	START;
	FADEIN;
	FADEOUT;
	RUNNING;
	LOSE;
	WIN;
}

class Level extends Scene
{

	public static var music: Sfx;
	public var state:GameState;
	public var fade: Float;

	public var player: Creature;
	public var targetZoom: Float = 1;
	public var zoom: Float = 1;

	public override function begin()
	{

		if(music==null) {
			music = new Sfx("audio/music.mp3");
			music.play(1, 0, true);
		}
		

		var map:Tiled = new Tiled("level.tmx");
		var g:Tilemap = map.createTilemap("Walls", "Tiles");
		addGraphic(g);


		addMask( map.createClipmap("Meta", "Meta", 0), "solid" );
		addMask( map.createClipmap("Meta", "Meta", 1), "duct" );

		// Doors
		for(d in map.getObjects("Door")) {
			add( new Door(d.x, d.y) );
		}

		// Enemies
		for(e in map.getObjects(null, "Enemies")) {
			if(e.type == "scientist") add(new Scientist(e.x, e.y) );
		}

		// Win trigger
		for(t in map.getObjects("trigger")) {
			var te:Entity = new Entity(t.x, t.y);
			te.setHitbox(Std.int(t.width), Std.int(t.height));
			te.type = t.name;
			add(te);
		}

		player = new Creature(300, 400);
		for(p in map.getObjects("player")) {
			player.x = p.x;
			player.y = p.y;
			break;
		}
		player.type = "character";
		add(player);

		state = START;
	}

	public override function update() {
		switch(state) {
		case START:
			fade = 1;
			if(Input.pressed(Key.ANY)) state = FADEIN;
			return;
		case RUNNING:
			if(Input.pressed(Key.R)) state = FADEOUT;
		case FADEIN:
			fade -= 0.03;
			if(fade <= 0) state = RUNNING;
		case FADEOUT:
			fade += 0.03;
			if(fade >= 1) HXP.scene = new Level();
		case LOSE:
			fade += 0.03;
			if(Input.pressed(Key.ANY) && fade>=1) state = FADEOUT;
			if(fade >= 1) return;
		case WIN:
			fade += 0.03;
			if(Input.pressed(Key.ANY) && fade>=1) state = FADEOUT;
			if(fade >= 1) return;
		}



		super.update();


		if(Input.pressed(Key.Z)) targetZoom = targetZoom == 1? 0.5: 1;
		zoom = HXP.lerp(zoom, targetZoom, 0.1);
		HXP.screen.scale = zoom;

		camera.x = player.x - HXP.halfWidth;
		camera.y = player.y - HXP.halfHeight;



		if(player.collideTypes(["win"], player.x, player.y)!=null) {
			state = WIN;
		}

		if(player.health <= 0) {
			state = LOSE;
			remove(player);
		}


	}

	public override function render() {
		super.render();

		// health bar
		Draw.rect(Std.int(camera.x + 20), Std.int(camera.y+20), Std.int((HXP.width-40) * player.health/100), 8, 0xff3000, 0.8);


		if(state != RUNNING) {
			Draw.setTarget(HXP.buffer);
			Draw.rect(0, 0, HXP.width, HXP.height, 0, fade);

			// Some text
			if(state == START) Draw.text("I need to get out of here ...", 100, 100);
			if(state == WIN) Draw.text("Freedom!\n\n\n\n\nPress any key to restart", 100, 100);
			if(state == LOSE) Draw.text("You died.\n\nMunch on scientists to gain health", 100, 100);

			Draw.resetTarget();
		}


	}
}
